# pert.awk
#
# Tches :  liste A=dbut B C ... Z=fin, nombre alatoire [ntmin, ntmax]
#           nombre de tches : nt                       
#           dures, alatoires  [dmin, dmax]
#           prcdences aucune ou avant dans l'ordre alphabtique
#                               [npmin, npmax]
#           dessin des tableaux
# Graphe :  Niveaux  dtermination des chemins critiques  dessin
# Gantt  :  dessin
# Utiliser le shell suivant
#
#  #!/bin/sh 
#  while [ ! "A$1" = "A" ] ; do
#  tname=tab$1".t"
#  base="gr-"$1
#  dname=$base".dot"
#  gname=$base".data"
#  pname=$base".ps"
#  echo ""|time gawk -v base=$base -f pert.awk > $tname
#  dot -Tps  $dname -o $pname
#  texif $tname      ## en-tte et fin de document ...
#  gv $tname".ps"&
#  shift
#  done    
########################################################################
### Choix des tches, des dures, des prcdentes
### calcule les suivantes
function datachoice() {
     nt = ntmin + int((ntmax-ntmin+1)*rand()) 
     for(i=0; i<= nt; i++)
       for(j=0; j<= nt; j++) {
         sliste[i, j] = 0;
         sstrict[i, j] = 0
         mat[i, j] = 0
       }
     for(i=0; i <= nt; i++) {
       dure[i] = dmin + int((dmax-dmin+1)*rand())
       np[i] = npmin + int((npmax-npmin+1)*rand())  # nb de precedents
       if(np[i] > i) np[i] = i
       k=0;
       while(k < np[i]) {
         t = int(i*rand())
         if(t<i && sliste[t, i] == 0) {
           mat[t, i] = 1
           nprec[i] += 1
           sliste[t, i] = 1
           sstrict[t, i] = 1
           nsucc[t] += 1
           k++
         }
       }
    }      
}
########################################################################
### Matrice de fermeture transitive
function matrix() {
  test = 1
  while(test==1) {
    test=0
    for(i=0; i< nt; i++)
      for(j=i+1; j<= nt; j++)
        for(k = i+1; mat[i, j] == 0 && k < j; k++)
          if(mat[i, k]==1 && mat[k, j]==1) {
            test = 1
            mat[i, j] = 1
          }
  }
}
########################################################################
### limine les raccourcis
###    si U -> V -> W  et U -> W alors mimine u -> W
function strict() {
  for(i=0; i< nt; i++)
    for(j=i+1; j<= nt; j++)
      for(k=i+1; sstrict[i, j]==1 && k<j; k++) 
        if(mat[i, k]==1 && mat[k, j] == 1) sstrict[i, j] = 0
}
########################################################################
###  crit le fichier source pour `dot'
###  ce fichier sera transform en image postscript par :
###  dot -Tps -o ?.ps ?.dot          
function writestrictdot() {
  print "digraph DG {" > fdot   # fdot contient le nom du fichier ?.dot
  print "  size=\"5,5\";"  > fdot
  print "  ranksep=0.3;"  > fdot
  print "  node [shape=\"plaintext\", fontsize=20];" > fdot
  for(i=0; i<= nvmax; i++) {
    printf("  {rank=same") > fdot
    for(j=0; j<= nt; j++) 
      if(niv[j]==i) printf(";%s",cara(j)) > fdot
    print "};" > fdot
  }
  for(i=0; i<= nt; i++) 
    for(j=0; j< i; j++) 
      if(sstrict[j, i] == 1) 
        if(marge[i]==0 && marge[j] == 0)
          printf("  %s -> %s;\n",cara(j),cara(i)) > fdot
        else
          printf("  %s -> %s [style=\"dotted\"];\n",cara(j),cara(i)) > fdot
  for(i=0; i<= nt; i++) {
    if(marge[i]==0) 
      print "  " cara(i) " [shape=\"circle\"];" > fdot
    # else print "  " cara(i) " [shape=\"plaintext\"];" 
  }
  print "}" > fdot
}                                                                               ########################################################################
### sortie latex du tableau des tches
function writetab() {
  print "\\begin{center}{\\bf Ordonnancement de tches}\\\\"
  print "Activits sur les sommets\\footnote"
  writecomment()
  print "\\end{center}"
  print "\\paragraph{Donnes et rsultats.}"
  print "{\\small Le tableau [{\\it fig.\\ \\ref{perttab}}] indique"
  print "les noms des tches  accomplir dans la colonne 2."
  print "Pour chacune des tches $t$, la dure est crite dans la colonne 3 "
  print "et les noms des tches qui doivent"
  print "tre termines avant que la tche $t$ ne commence sont indiqus"
  print "dans la colonne 4.\\\\"
  print "Les valeurs des colonnes suivantes sont calcules  partir "
  print "des donnes prcdentes.\\\\"
  print "Lorsque la marge est nulle, la tche est critique, retarder"
  print "les dates de son excution serait retarder la dure de tout"
  print "le projet.\\\\"
  print "Si toutes les tches sont excutes entre leurs"
  print "dates de dbut au plus tt et de fin au plus tard,"
  print "la dure du projet est de $" fmax "$ units de temps.}\\\\"
  print "\\begin{figure}[ht]"
  print "\\begin{center}"
  print "  {\\scriptsize"
  print "  \\begin{tabular}{| r | c | c | c | c | c | c | c | c | c | c |}"
  print "  \\hline"
  printf (" & Tche & Dure  & Prcdentes & Suivantes & Niveau & Marge & ")
  print   "Dbut au  & Fin au & Dbut au  & Fin au\\\\"
  printf ("$n$ & $t_n$ & $d_n$ & & & $r_n$ &$m_n$ & plus tt & plus tt &")
  print   " plus tard & plus tard \\\\"
  print "  \\hline"
  for(i = 0; i<= nt; i++) {
    printf("%d & %s & %d & ", i+1, caratex(i), dure[i])
    t = 0
    for(j=0; j<i; j++)
      if(sstrict[j, i]==1) t++
    for(j=0; j<i; j++) {
      if(sstrict[j, i]==1) {
        t--
        printf("%s",caratex(j))
        if(t>0) printf(",")
        printf(" ")
      }
    }    
    printf (" & ")
    t = 0
    for(j=i+1; j<= nt; j++) 
      if(sstrict[i, j]==1) t++
    for(j=i+1; j<= nt; j++) {
      if(sstrict[i, j]==1) {
        t--
        printf("%s",caratex(j))
        if(t>0) printf(", ")
        else printf(" ")
      }
    }
 print "&"niv[i]"&"marge[i]"&"dptot[i]"&"fptot[i]"&"dptard[i]"&"fptard[i]"\\\\"
    print "  \\hline"
  }
  print "  \\end{tabular}"
  print "  }"
  print "\\caption{\\label{perttab}{\\it Tableau des tches}}"
  print "\\end{center}"
  print "\\end{figure}"
}           
########################################################################
###  commandes gnuplot pour produire le diagramme de Gantt
function writedata() {
  for(i=0; i<= nt; i++) {
    print i+1 " " fptard[i] " " dptot[i] " " fptot[i] > fdata
  }
  s = "echo \"set terminal post portrait \\\"Times-Roman\\\" 14;"
  s = s " set nokey; set grid ;"
  s = s "set output \\\"" epost "\\\";plot \\\"" fdata "\\\" with errorbars;\"|gnuplot"
  system(s)
}
########################################################################
###  sortie latex pour la figure du graphe
function writefigdot() {
  print "\\paragraph{Graphe.}"
  print "\\begin{figure}[ht]"
  print "\\begin{center}"
  print "\\hspace*{0cm}\\psfig{figure=" post ",height=19cm,clip=}"
  print "\\caption{\\label{diag}{\\it Diagramme des tches.}}"
  print "\\end{center}"
  print "\\end{figure}"
  print "{\\small La figure [{\\it fig.\\ \\ref{diag}}] donne le graphe de la"
  print "relation de prcdence entre les diffrentes tches.\\\\"
  print "Les $" nvmax+1 "$ niveaux indiqus dans le tableau"
  print "[{\\it fig.\\ \\ref{perttab}}] se retrouvent sur le graphe,"
  print "ils vont du niveau $0$, en haut de la figure, au plus"
  print "grand niveau $" nvmax "$, en bas."
  print "Les tches critiques, dont les noms sont cercls, "
  print "sont disposes du niveau $0$ au niveau $" nvcmax "$,"
  if(nvcmax<nvmax)
  print "(infrieur au niveau maximum $" nvmax "$ dans ce cas particulier),"
  else {
  print "(mais qui pourrait tre infrieur au niveau maximum dans un"
  print "autre exemple)."
  }
  print "Les arcs ne reliant pas deux tches critiques sont"
  print "dessins en pointills.}"
}
########################################################################
###  sortie latex pour la figure du diagramme de Gantt
function writefigplot() {
  print "\\paragraph{Diagramme de Gantt.}"
  print "\\begin{figure}[ht]"
  print "\\begin{center}"
  print "\\hspace*{0cm}\\psfig{figure=" epost ",height=19cm,clip=}"
  print "\\caption{\\label{gantt}{\\it Diagramme de Gantt.}}"
  print "\\end{center}"
  print "\\end{figure}"
  print "{\\small La figure [{\\it fig.\\ \\ref{gantt}}] donne le diagramme de"
  print "Gantt. Pour chacune des tches, le segment indique les"
  print "dates de dbut au plus tt et de fin au plus tt, la "
  print "marque $+$ indique la date de fin au plus tard."
  print "Lorsque la tche est critique, la marque $+$ est "
  print "l'extrmit suprieure du segment. La valeur de la marge"
  print "est le laps de temps compris entre l'extrmit suprieure"
  print "du segment et le signe $+$.}"
}
########################################################################
### sortie latex du commentaire
function writecomment() {
  print "{L'exemple  (donnes, traitement et solution)"
  print "a t ralis automatiquement par des moyens informatiques,"
  print "il ne correspond pas  un problme rel d'ordonnancement.\\"
  print "Le programme principal d'environ $300$ lignes "
  print "(gnration alatoire des donnes,"
  print "calcul des niveaux, des tches critiques, des dates et des"
  print "marges) est crit en AWK. Ce programme permettrait le traitement"
  print "de problmes plus importants (mais la dure varie comme le cube"
  print "du nombre de tches).\\\\"
  print "Les fichiers et commandes produits par le programme principal sont"
  print "\\begin{itemize}"
  print "\\item un fichier dans le langage de DOT pour l'obtention"
  print "du diagramme sagittal du graphe,"
  print "\\item un fichier de donnes pour le diagramme de Gantt et"
  print "une commande de traitement par GNUPLOT,"
  print "\\item un fichier \\LaTeX\\ qui donne le document final."
  print "\\end{itemize}"
  print "Citons parmi les outils mathmatiques et"
  print "les algorithmes utiliss, la"
  print "recherche de la fermeture transitive du graphe avant"
  print "l'limination de tous les raccourcis qui ne feraient que"
  print "rendre moins lisible le graphe.}"
}
########################################################################
### dtermine les niveaux des sommets
function niveau() {
   nvmax = 0
  for(i=0; i<= nt; i++) {
    niv[i] = 0;
    for(j=0; j< i; j++) {
      v = niv[j]+1
      if(sstrict[j, i]==1 && v > niv[i]) niv[i] = v
    }
    if(niv[i] > nvmax) nvmax = niv[i]
  }
}
########################################################################
### calcul du dbut et de la fin au plus tt
function finaptot() {
  for(i=0; i<= nt; i++) {
    dt = 0
    for(j=0; j< i; j++) 
      if(sliste[j, i] == 1) 
        if(fptot[j] > dt) dt=fptot[j] 
    dptot[i] = dt
    fptot[i] = dt + dure[i]
    if(fptot[i] > final) final = fptot[i] 
  }
}
########################################################################
### calcul du dbut et de la fin au plus tard
function finaptard() {
  fmax = 0
  for(i=nt; i>=0; i--) {
    dt = final
    for(j=nt; j> i; j--) 
      if(sliste[i, j] == 1) 
        if(dptard[j] < dt) dt=dptard[j] 
    fptard[i] = dt
    dptard[i] = dt - dure[i]
    if(fptard[i]> fmax) fmax = fptard[i]
  }
}
########################################################################
### niveau max des tches critiques
function nvcrit() {
  nvcmax = 0
  for(i=0; i<= nt; i++)
    if(marge[i]==0 && niv[i]>nvcmax) nvcmax = niv[i]
}
########################################################################
###  marge (diff entre dbuts au plus tt et au plus tard
function marges() {for(i=0; i<= nt; i++) marge[i] = dptard[i]-dptot[i]}
########################################################################
###  conversion nombre -> caractre
function cara(n) {
  return("t" (n+1))
}
########################################################################
###  conversion nombre -> caractre ###  idem mais pour LaTeX
function caratex(n) {
  return("$t_{" (n+1) "}$")
}                                                                               
########################################################################
### programme principal
###
END {if(npmin=="") npmin=0
     if(npmax=="") npmax=npmin+10
     if(ntmin=="") ntmin=28 #289
     if(ntmax=="") ntmax=ntmin+0
     if(dmin=="") dmin=2
     if(dmax=="") dmax=20*dmin
     if(base == "") base = "gr-" 
     fdot = base ".dot"; post = base ".ps"
     epost = base ".eps"; fdata = base ".data"
     srand(); datachoice(); 
     matrix(); strict(); niveau();
     finaptot(); finaptard(); marges(); nvcrit();
     writedata(); 
     writestrictdot();
     writetab(); 
     writefigdot(); 
     writefigplot();
}     
